." Text automatically generated by txt2man
.TH uchess 1 "January 06, 2021" "uchess" "uchess Reference Manual"
.SH NAME
\fBuchess \fP- terminal user interface for UCI chess engines.
.SH SYNOPSIS
.nf
.fam C
\fBuchess\fP [\fB-black\fP \fIplayer\fP] [\fB-white\fP \fIplayer\fP] [\fB-cfg\fP \fIconfig\fP] [\fB-themes\fP] [\fB-tmpl\fP]
.fam T
.fi
.SH DESCRIPTION
\fBuchess\fP is an interactive terminal chess client designed to allow
gameplay and move analysis in conjunction with UCI chess engines.
.PP
\fBuchess\fP is highly configurable, fully themeable, and capable of
supporting a variety of play scenarios including human vs. human,
human vs. cpu, and cpu vs. cpu.
.PP
Games may be initialized via FEN notation and exported to PGN notation
for further analysis in third-party engines. Additionally, board
snapshots may be saved to SVG image format for the purpose of sharing
or archiving board state.
.PP
A hint engine is provided to assist players in discovering the best
move in a given scenario and game state may be walked backward on an
as-needed basis to facilitate game tree exploration. 
.PP
Game moves are input as algebraic notation via an interactive command
shell, and a variety of auxillary commands are supported.
.SH EXAMPLE
Generate a \fBuchess\fP \fIconfig\fP and edit it accordingly.
.PP
.nf
.fam C
      $ uchess -tmpl > uchess.json

.fam T
.fi
Run \fBuchess\fP on the \fIconfig\fP.
.PP
.nf
.fam C
      $ uchess -cfg uchess.json

.fam T
.fi
With zero configuration, \fBuchess\fP will attempt to invoke stockfish
via /usr/local/bin/stockfish with difficulty level 20.
.SH OPTIONS
.TP
.B
\fB-help\fP
Display command usage.
.TP
.B
\fB-black\fP
Black piece input <cpu|human>.
.TP
.B
\fB-white\fP
White piece input <cpu|human>.
.TP
.B
\fB-cfg\fP
Path to \fIconfig\fP file.
.TP
.B
\fB-tmpl\fP
Write default \fIconfig\fP to stdout and exit.
.TP
.B
\fB-themes\fP
Write theme names to stdout and exit.
.SH SHELL COMMANDS
\fBuchess\fP accepts commands via an interactive shell. The following commands
are supported.
.TP
.B
back
Walk the game state back one turn.
.TP
.B
save
Save the PGN for the current game in the CWD.
.TP
.B
image
Save an SVG snapshot of the current game in the CWD.
.TP
.B
fen
Display the FEN string for the current game.
.TP
.B
reset
Reset the board to the default FEN.
.TP
.B
resign
The current \fIplayer\fP resigns.
.TP
.B
hint
Highlight the recommended move using the hint engine.
.TP
.B
quit
Shutdown \fBuchess\fP immediately without saving game state.
.PP
If none of the previous commands are recognized, the input is assumed
to be a move specified in algebraic notation.
.PP
Examples:
.RS
.TP
.B
e4
Pawn to e4.
.TP
.B
exd5
Pawn on the e-file captures the piece on d5.
.TP
.B
O-O
King castles on kingside.
.TP
.B
O-O-O
King castles on queenside.
.TP
.B
e8=Q
Pawn promotion to queen.
.SH BASE CONFIG FORMAT
When invoked with the \fB-tmpl\fP argument, \fBuchess\fP will generate a \fIconfig\fP with
a reasonable set of defaults. The \fIconfig\fP file is a JSON document with
the following top-level keys.
.TP
.B
uciWhite
Name of the UCI engine controlling white pieces.
.TP
.B
uciBlack
Name of the UCI engine controlling black pieces.
.TP
.B
uciHint
Name of the UCI engine providing hints.
.TP
.B
uciEngines
A list of available UCI engines (see UCI CONFIG FORMAT).
.TP
.B
fen
FEN notation specifying starting positions.
.TP
.B
activeTheme
The currently selected theme (list available via \fB-themes\fP).
.TP
.B
theme
A list of available themes (see THEMES).
.TP
.B
whitePiece
Player controlling the white pieces (cpu or human).
.TP
.B
blackPiece
Player controlling the black pieces (cpu or human).
.TP
.B
whiteName
Player name for white pieces in UI.
.TP
.B
blackName
Player name for black pieces in UI.
.SH UCI CONFIG FORMAT
The \fBuchess\fP \fIconfig\fP file may reference any number of UCI engines; however,
each engine must by identified by a unique name parameter. The following
keys are supported for UCI configuration.
.TP
.B
name
The unique UCI engine name.
.TP
.B
engine
The path to the UCI engine binary (OS dependent).
.TP
.B
hash
The value in MB for hash table memory.
.TP
.B
ponder
Allow or disallow the engine to ponder.
.TP
.B
ownBook
Specifies whether the engine has its own opening book.
.TP
.B
multiPV
The engine supports multi best line or k-best mode.
.TP
.B
depth
Search x plies only. A setting of 0 specifies unlimited plies.
.TP
.B
searchMoves
Restrict search to moves specified (i.e., "e2e4 d2d4").
.TP
.B
moveTime
Search exactly x mseconds.
.TP
.B
options
Key-value pairs for arbitrary engine commands.
.SH THEMES
\fBuchess\fP is fully themeable, and user specified themes may be added to the
\fBuchess\fP \fIconfig\fP file. The theme keys are named in a manner which is intended to
be self-explanatory, and colors are specified by their respective hex
values. It is recommended to limit colors to the spectrum supported by
your terminal. xterm-256 (8-bit color) is the official standard for
builtin themes. 
.PP
The builtin themes are packaged into the \fBuchess\fP binary using pkger, and
it is necessary to invoke this command before compilation when a theme
is added to the themes project directory.
.PP
Pull requests for new builtin themes are welcome; however, the specified 
colors must fall under the xterm-256 standard. A color chart is available here:
.PP
https://en.wikipedia.org/wiki/Xterm#/media/File:Xterm_256color_chart.svg
.PP
Builtin themes should be specified on a one theme per file basis, and all
themes should be specified in JSON format and reside in the themes directory.
.PP
When a name collision occurs, themes specified in the \fIconfig\fP file will
override builtin themes.
.PP
Lastly, a special hex code of #0 is used to specify the terminal default
color. This code should be used for any UI elements that may collide with
an underlying color scheme (i.e., avoiding white fonts on white backgrounds).
.SH GAME OUTCOMES
\fBuchess\fP can effectively identify a wide variety of game outcomes, and it should
account for the following end-game scenarios:
.PP
.nf
.fam C
    Checkmate
    Stalemate
    Resignation (manual)
    Threefold Repetition
    Fivefold Repetition
    Fifty Move Rule
    Seventy Five Move Rule
    Insufficient Material
.fam T
.fi
.SH CPU MATCHES
If the \fBuchess\fP \fIconfig\fP specifies both whitePiece and blackPiece as cpu,
the specified UCI engines will play against each other. The game will
cycle forward one move each time tne enter key is pressed.
.SH PLATFORM SUPPORT
\fBuchess\fP has been tested and confirmed to work on Linux, MacOS, and Windows
(PowerShell) platforms. It should work with a wide variety of terminals.
.SH BUGS
Some UCI chess engines currently fail due to a lack of proper initialization.
This will be resolved in a future release. For the moment, stockfish is the
only UCI engine that is officially supported.
.SH AUTHOR
Travis Whitton <tinymountain@gmail.com>
